//////////////////////////////////////////////////////////////////////////////
//
//  (C) Copyright Ladislav Sopko, 3D Informatica s.r.l 2014
//  
//////////////////////////////////////////////////////////////////////////////
#if defined(_MSC_VER)&&(_MSC_VER>=1200)
#pragma once
#endif


#ifndef XW_IPC_ROBUST_MUTEX_FIXER_HPP
#define XW_IPC_ROBUST_MUTEX_FIXER_HPP


#include <boost/shared_ptr.hpp>
#include <xw-incl/ipc/exception.hpp>
#include <boost/noncopyable.hpp>
#include <xw-incl/ipc/named_robust_mutex.hpp>

#ifdef strRev
#	undef strRev
#endif
#define strRev "$Revision: 1.3 $"

#define IPC_MAKE_FIXER(type) xw::ipc::mutex_fixer_ptr_t(type::make())

namespace xw{ namespace ipc{

	class IMutexFixer : boost::noncopyable
	{
	public:
		virtual bool fix(void) = 0;
	};

	typedef boost::shared_ptr<IMutexFixer> mutex_fixer_ptr_t;

	class dummy_fixer : public IMutexFixer {
	public:
		virtual bool fix(){ 
			IPC_DBG_TRACE("dummy fixer done.");
			return true; 
		}
		static dummy_fixer* make(){
			return new dummy_fixer();
		}
	}; 

	template<typename exceptionT>
	class exception_fixer : public IMutexFixer{
	public:
		virtual bool fix(){ 
			XW_THROW_EXCEPTION(exceptionT, "System must be fixed!");
		}
		static exception_fixer<exceptionT>* make(){
			return new exception_fixer<exceptionT>();
		}
	}; 

	typedef exception_fixer<ex_ipc_fixer_generic>	ex_ipc_fixer_generic_exception_fixer_t;


}}  //xw::ipc



#endif
